//    Copyright (C) 2004 Michael Mahr

//    This file is part of "lesschess".
//
//    "lesschess" is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    "lesschess" is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with "lesschess"; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "ascmodel.h"

AscModel::AscModel(std::string filename)
{
  std::ifstream ascfile(filename.c_str());
  
  if (!ascfile.is_open())
  {
    throw AscException("Ascfile not found");
  }
  
  char linebuffer[4096];

  int mode = 0;

  while (!ascfile.eof())
  {
    ascfile.getline(linebuffer, 4096);
    std::string line(linebuffer);
    
    if (line.substr(0,12) == "Vertex list:")
    {
      mode = 1;
      continue;
    }
    else if (line.substr(0,10) == "Face list:")
    {
      mode = 2;
      continue;
    }
    
    //processing of the modes
    switch (mode)
    {
      case 1:
        if (!(line.substr(0,6)=="Vertex"))
	{
	  std::cout << "AscModel:Unexpected end of verticle-list" << std::endl;
	  mode = 0;
	}
	else
	{
	  unsigned int position = atoi(line.substr(line.find_first_of(' '), line.find_first_of(':')-line.find_first_of(' ')).c_str());
	  
	  std::string rawx = line.substr(line.find_first_of('X')+3, line.find_first_of('Y')-line.find_first_of('X')-3);
	  rawx = rawx.replace(rawx.find(","),1,".");
	  std::string rawy = line.substr(line.find_first_of('Y')+3, line.find_first_of('Z')-line.find_first_of('Y')-3);
	  rawy = rawy.replace(rawy.find(","),1,".");
	  std::string rawz = line.substr(line.find_first_of('Z')+3, line.size()-line.find_first_of('Z')-3);
	  rawz = rawz.replace(rawz.find(","),1,".");
	  
	  AscVector tempvec;
	  tempvec.x = atof(rawx.c_str());
	  tempvec.y = atof(rawy.c_str());
	  tempvec.z = atof(rawz.c_str());
	  
	  
	  verticles[position] = tempvec;
	}
        break;
      case 2:
        if (line.substr(0,4)=="Face")
	{
	  unsigned int position = atoi(line.substr(line.find_first_of(' '), line.find_first_of(':')-line.find_first_of(' ')).c_str());
	  
	  AscFace tempface;
	  
	  tempface.a = atoi(line.substr(line.find_first_of('A')+2, line.find_first_of('B')-line.find_first_of('A')-2).c_str());
	  tempface.b = atoi(line.substr(line.find_first_of('B')+2, line.find_first_of('C')-line.find_first_of('B')-2).c_str());
	  tempface.c = atoi(line.substr(line.find_first_of('C')+2, line.size()-line.find_first_of('C')-2).c_str());

          //std::cout << tempface.a << "  " << tempface.b << "  " << tempface.c << std::endl;
	  
	  faces[position] = tempface; 

	}
	else if (line.substr(0,10)=="Smoothing:")
	{
	  //smoothing is ignored - calculated by gl
	}
	else if(line == "")
	{} //empty lines are ignored
	else
	{
	  std::cout << "AscModel:Unexpected end of face-list" << std::endl;
	  std::cout << line << std::endl;
	  mode = 0;
	}
        break;
    }
    
  }  
  
}

AscModel::~AscModel()
{}
