//    Copyright (C) 2004 Michael Mahr

//    This file is part of "PS2L_UtilClasses".
//
//    "PS2L_UtilClasses" is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    "PS2L_UtilClasses" is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with "PS2L_UtilClasses"; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef __matrixmanager__
#define __matrixmanager__

//for memset
#include <string>

//for atan
#include <math.h>


//modelview:   local->world ;up to 16 matrices
//perspective: world->clip  ;1 matrix
//screenfit:   clip->screen ;1 matrix

//note: _you_ _must_ ensure that the instance of this class is 128-bit aligned
//do not change the order of the class-members!

//you cannot modify an existing matrix on the stack - thats on purpose
//instead simply push on a second matrix too and pop twice when done


class MatrixManager
{
public:
  MatrixManager();
 
  //adds or removes matrices from the modelview-stack
  void modelviewPush(float * matrix);
  void modelviewPop();
  
  //puts a new scale, translate or rotation matrix onto the stack
  void modelviewPushTranslate(float x, float y, float z);         //moves an object by xyz
  void modelviewPushScale(float x, float y, float z);             //scales an object by xyz
  void modelviewPushRotate(float x, float y, float z, float deg); //rotates an object deg degrees around the axis xyz
  
  void perspective(float * matrix);
  //generates an infinite projection matrix (unless you use fog this works better than a finite one: I only say z-precission and stencilshadows)
  void perspectiveInfinite(float znear, float left, float right, float top, float bottom);
  void perspectiveInfinite(float znear, float aspect, float fov);

  void screenfit(float * matrix);
  
  const float * gettransform(); //returns the local-to-screen transform  
  const float * getclipspacetransform(); //returns the local-to-clipspace transform
  
  //here ends the "normal" functionality of the class
  //the following functions should only be used for testing
  
  //note: you have to ensure that the float[]s you pass are 128-bit aligned
  void loadmatrix(); //loads the current transform into the vu0 registers
  void loadclipspacematrix(); //loads the current clipspacetransform into the vu0 registers
  void facetransform(void * destface, void * face);      //transforms 3*vertex=12*float 
  void facetransformi4(void * destface, void * face);     //transforms 3*vertex=12*float - also converts the values to fixed point integers suitable for the gs
  void vertextransform(void * destvertex, void * vertex);      //transforms 1*vertex=4*float 
  void vertextransformi4(void * destvertex, void * vertex);     //transforms 1*vertex=4*float - also converts the values to fixed point integers suitable for the gs


private:
  float modelviewmatrix[16][16];  //the maximum of 16 modelview matrices are supported
  
  float perspectivematrix[16]; //one perspective matrix (should be sufficient in most cases)
  float screenfitmatrix[16];   //one screenfitmatrix

  float modelviewtransform[16][16]; //old transforms are kept avalilable
  float perspectivescreenfittransform[16]; //a combination of perspective and screenfit
  float finaltransform[16]; //the final transform

  float clipspacetransform[16]; //stores the cliptransform for... well... clipping

  unsigned char modelviewtop; //pointer to the topmost modelviewmatrix
  
  bool finalchanged;
  bool clipspacechanged;
  
  void matrixproduct(float * output, const float * inputa, const float * inputb);
  void identity(float * matrix);  
  
};

#endif
