
#include <iostream>
#include "matrixmanager.h"
#include <sps2lib.h>
#include <sps2tags.h>
#include <sps2util.h>

MatrixManager matrixmanager;

void displaymatrix(const float * m)
{
  std::cout << m[0] << "\t" << m[4] << "\t"<< m[8] << "\t" << m[12] << std::endl;
  std::cout << m[1] << "\t" << m[5] << "\t"<< m[9] << "\t" << m[13] << std::endl;
  std::cout << m[2] << "\t" << m[6] << "\t"<< m[10] << "\t" << m[14] << std::endl;
  std::cout << m[3] << "\t" << m[7] << "\t"<< m[11] << "\t" << m[15] << std::endl;
}

void displayface(const float * m)
{
  std::cout << "vertex a:    x: " << m[0] << "\t    y: " << m[1] << "\t    z: "<< m[2] << "\t w:" << m[3] << std::endl;
  std::cout << std::endl;
  std::cout << "vertex b:    x: " << m[4] << "\t    y: " << m[5] << "\t    z: "<< m[6] << "\t w:" << m[7] << std::endl;
  std::cout << std::endl;
  std::cout << "vertex c:    x: " << m[8] << "\t    y: " << m[9] << "\t    z: "<< m[10] << "\t w:" << m[11] << std::endl;

}

void displayface(const unsigned int * m)
{
  std::cout << "vertex a: "<< (m[0]>>4) << "\t" << (m[1]>>4) << "\t"<< (m[2]>>4) << "\t" << (m[3]) << std::endl;
  std::cout << std::endl;
  std::cout << "vertex b: " << (m[4]>>4) << "\t" << (m[5]>>4) << "\t"<< (m[6]>>4) << "\t" << (m[7]) << std::endl;
  std::cout << std::endl;
  std::cout << "vertex c: " << (m[8]>>4) << "\t" << (m[9]>>4) << "\t"<< (m[10]>>4) << "\t" << (m[11]) << std::endl;
}


int main(int argc, char ** argv)
{
  int sps2Descriptor = sps2Init();

  //create the projectionmatrix
  //this tells the matrixmanager how to transform from world-space to clip-space
  matrixmanager.perspectiveInfinite(1.0f, 720.0f/576.0f, 90);
  //1.0 is the near clipping plane, 720/576 is the aspect ratio and 90 is the field of view for vertical; the horizontal fov is calculated using the aspect ratio
  //this also creates a clip-space (frustum) which is quite handy for clipping: x:-1/1 y:-1/1 z:-1/1


  //create and set a screenfitmatrix (the following matrix is an example for PS2)
  //this tells the matrixmanager how to transforms from clip-space to screen-space
  float screenfitmatrix[] = {720,0,0,0,0,576,0,0,0,0,-65535/2,0,2048,2048,65535/2,1};
  //720 is the width, 576 is the height and 65535 is the z-depth at the near clipping plane, 0 is the zdepth at infinity
  //2048/2048 is the center of the screen
  //this matrix scales the clip-space by (720, 576, 65535/2) and afterwards shifts it by (2048, 2048, 65535/2)
  
  matrixmanager.screenfit(screenfitmatrix);

  //these tell the matrixmanager how to transform from local-space to world-space
  matrixmanager.modelviewPushTranslate(0,0,-6);  //move object 6 units into the screen (you have to use minus here! Otherwise clipping won't work)
  matrixmanager.modelviewPushScale(2,2,2);       //scale object by factor of 2  


  //a face - "__attribute__((aligned(16)))" aligns the array so vu0 can read it
  float face[] __attribute__((aligned(16))) = {-3,-3,0,1,-3,3,0,1,3,3,0,1};
  float dest[12] __attribute__((aligned(16))); //tempoary container for transformed data

  /*clipping judgement*/

  //calculate (if necessary) and load the current clipspacematrix into vu0
  matrixmanager.loadclipspacematrix();
    
  matrixmanager.facetransform(dest, face); //<- here all the magic happens!
  //transforms the face and converts homogenous coordinated to orthogonal ones (using clipspace)

  std::cout << std::endl;
  std::cout << "clip-space" << std::endl;  
  displayface(dest); //displays the face-data in clip-space coordinates
  //the w-coordinate can be ignored since it already was processed during "facetransform"
  
  //0.8 and -0.8 means that this coord is on screen.
  //1.0 and -1.0 means that it is at the border of the screen.
  //everything greater than 1.0 or less than -1.0 can be clipped


  /*drawing*/

  //calculate (if necessary) and load the current transformationmatrix into vu0
  matrixmanager.loadmatrix();
    
  matrixmanager.facetransform(dest, face); //<- here all the magic happens!
  //transforms the face and converts homogenous coordinated to orthogonal ones (using final(screen) transform)
  
  std::cout << std::endl;
  std::cout << "screen-space" << std::endl;  
  displayface(dest); //displays the face-data in screen coordinates
  //the w-coordinate can be ignored since it already was processed during "facetransform"

  //these coords can be directly sent to the GS


  //take the scale, rotation and translation from the stack again
  matrixmanager.modelviewPop();
  matrixmanager.modelviewPop();
  
  
  sps2Release(sps2Descriptor);
}

